@@ -1,16 +1,22 @@
+from __future__ import annotations
+
+from typing import TYPE_CHECKING, Type
+
 from ..decorators import closure, delete, get
 from ..error import BadRequest
 from ..generic import Map, Tuple
+from ..json import to_json
 from ..scalar.bound import Range
 from ..scalar.number import Bool, UInt
-from ..scalar.ref import form_of, If, Ref
+from ..scalar.ref import If, Ref, form_of
 from ..state import State, Stream
-from ..uri import uri, URI
-from ..context import to_json
-
-from .base import Collection
+from ..uri import URI
+from .base import Collection, Column
 from .btree import BTree
 
+if TYPE_CHECKING:
+    from ..app import Model
+
 
 class Schema(object):
     """A `Table` schema which comprises a primary key and value :class:`Column` s."""
@@ -34,7 +40,7 @@ class Schema(object):
 class Table(Collection):
     """A `Table` defined by a primary key, values, and optional indices."""
 
-    __uri__ = uri(Collection) + "/table"
+    __uri__ = URI(Collection) + "/table"
 
     def __getitem__(self, key):
         """Return the row with the given key, or a :class:`NotFound` error."""
@@ -53,7 +59,8 @@ class Table(Collection):
         @closure(self)
         @get
         def group(cxt, key: Tuple) -> Tuple:
-            cxt.where = Tuple(columns).zip(key).cast(Map)
+            cxt.columns = columns
+            cxt.where = cxt.columns.zip(key).cast(Map)
             cxt.slice = self.where(cxt.where)
             return key, fn(cxt.slice)
 
@@ -85,7 +92,7 @@ class Table(Collection):
         """
 
         delete_row = delete(lambda cxt, key: self.delete_row(key))
-        if uri(self).id():
+        if URI(self).id():
             delete_row = closure(self)(delete_row)
 
         to_delete = self.where(where) if where else self
@@ -201,3 +208,35 @@ def _handle_bounds(bounds):
         col: Range.from_slice(bounds[col]) if isinstance(bounds[col], slice) else bounds[col]
         for col in bounds
     }
+
+
+def create_schema(modelclass: Type[Model]) -> Schema:
+    """Create a table schema for the given model. A key for the table is auto
+    generated using the `class_name` function, then suffixed with '_id'. Each
+    attribute of the model will be considered as a column if it is of type
+    Column or Model.
+    """
+    values = []
+    indices = []
+    base_attributes = set()
+
+    for b in modelclass.__bases__:
+        base_attributes |= set(dir(b))
+
+    for f in base_attributes ^ set(dir(modelclass)):
+        attr = getattr(modelclass, f)
+        if isinstance(attr, Column):
+            values.append(attr)
+        else:
+            try:
+                from ..app import Model, class_name
+                assert issubclass(attr, Model)
+                values.append(*attr.key())
+                indices.append((class_name(attr), [attr.key()[0].name]))
+            except (TypeError, AssertionError):
+                continue
+
+    schema = Schema(modelclass.key(), values)
+    for i in indices:
+        schema.create_index(*i)
+    return schema
