@@ -1,16 +1,25 @@
+from __future__ import annotations
+
+from typing import TYPE_CHECKING, Type
+
 from ..app import App
 from ..chain import Sync
 from ..collection import Column
 from ..collection.table import Table
+from ..collection.table import create_schema as cts
 from ..collection.tensor import Sparse
-from ..error import BadRequest
 from ..decorators import closure, get, put
+from ..error import BadRequest
 from ..generic import Tuple
-from ..scalar.number import Bool, U32
+from ..scalar.number import U32, Bool
 from ..scalar.ref import After, If, Put
-from ..uri import uri
-
+from ..uri import URI
 from .edge import DIM, Edge, ForeignKey
+from .edge import Schema as EdgeSchema
+
+if TYPE_CHECKING:
+    from ..app import Model
+    from ..collection.table import Schema as TableSchema
 
 ERR_DELETE = "cannot delete {{column}} {{id}} because it still has edges in the Graph"
 
@@ -76,8 +85,8 @@ class Graph(App):
     are automatically updated when a `Table` is updated, but `Edge` relationships require explicit management.
     """
 
-    # TODO: remove the `schema` and `chain_type` parameters and generate the initial schema via reflection
-    def __init__(self, schema, chain_type=Sync):
+    # TODO: remove the `chain_type` parameter.
+    def __init__(self, schema: Schema, chain_type=Sync):
         for (label, edge) in schema.edges.items():
             if hasattr(self, label):
                 raise IndexError(f"{label} is already reserved in {self} by {getattr(self, label)}")
@@ -95,9 +104,14 @@ class Graph(App):
 
         App.__init__(self)
 
+    @classmethod
+    def autogenerate(cls, models: list[Model], chain_type=Sync):
+        """Auto create the schema and initalise it using a list of models."""
+        return cls(schema=_initalise_schema(models), chain_type=chain_type)
+
 
 def graph_table(graph, schema, table_name):
-    if uri(graph).startswith("/state"):
+    if URI(graph).startswith("/state"):
         raise RuntimeError("Graph requires an absolute URI--set this using your subclass's __uri__ attribute")
 
     table_schema = schema.tables[table_name]
@@ -135,7 +149,7 @@ def graph_table(graph, schema, table_name):
         args = closure(new_id)(get(lambda row: [new_id, Tuple(row)[0]]))
 
         # assumes row[0] is always the key
-        add = put(lambda new_id, key: Put(uri(adjacent), [new_id, key], True))
+        add = put(lambda new_id, key: Put(URI(adjacent), [new_id, key], True))
         add = to_table.where({edge.column: new_id}).rows().map(args).for_each(add)
 
         return After(If(cond, delete_row(edge, adjacent, row)), add)
@@ -209,3 +223,27 @@ def graph_table(graph, schema, table_name):
             return After(Table.upsert(self, key, values), updates)
 
     return GraphTable(table_schema)
+
+
+def _initalise_schema(models: list[Model]):
+    """Automatically build a Graph of all models."""
+    return create_schema([cts(m) for m in models])
+
+
+def create_schema(schemas: list[TableSchema]) -> Schema:
+    """Create a graph schema using model schemas."""
+    graph_schema = Schema()
+    indices = {}
+    from ..collection.table import Schema as TableSchema
+
+    for s in schemas:
+        assert isinstance(s, TableSchema) and isinstance(s.key[0], Column)
+        name = s.key[0].name.removesuffix("_id")
+        graph_schema.create_table(name, s)
+        for i in s.indices:
+            indices[f"{name}_{i[0]}"] = (f"{i[0]}.{i[1][0]}", f"{name}.{i[1][0]}")
+
+    for k, v in indices.items():
+        graph_schema.create_edge(k, EdgeSchema(*v))
+
+    return graph_schema
