@@ -6,7 +6,7 @@ from .interface import Numeric
 def is_numeric(state):
     """Return `True` if the given `State` is :class:`Numeric` or a numeric literal, otherwise `False`."""
 
-    return isinstance(state, (bool, float, int, Numeric))
+    return isinstance(state, (bool, complex, float, int, Numeric))
 
 
 # TODO: add a generic type to `Functional` and return an instance of that type
