@@ -1,8 +1,10 @@
 import inspect
 import logging
 
-from ..uri import uri, URI
-from ..context import to_json
+from ..json import to_json
+from ..uri import URI
+
+from .stub import MethodStub
 
 
 class Meta(type):
@@ -14,7 +16,7 @@ class Meta(type):
         parents = []
         for parent in cls.mro()[1:]:
             if issubclass(parent, State):
-                if uri(parent) != uri(cls):
+                if URI(parent) != URI(cls):
                     parents.append(parent)
 
         return parents
@@ -59,7 +61,9 @@ class Meta(type):
 
                 form[name] = attr
             if isinstance(attr, MethodStub):
-                form[name] = to_json(attr.method(instance_header, name))
+                # TODO: resolve these in alphabetical order
+                for method_name, method in attr.expand(instance_header, name):
+                    form[method_name] = to_json(method)
             else:
                 form[name] = attr
 
@@ -71,19 +75,7 @@ class Meta(type):
 
         parents = cls.parents()
 
-        if not parents or uri(parents[0]).startswith(uri(State)):
-            return {str(uri(Class)): to_json(form_of(cls))}
+        if not parents or URI(parents[0]).startswith(URI(State)):
+            return {str(URI(Class)): to_json(form_of(cls))}
         else:
-            return {str(uri(parents[0])): to_json(form_of(cls))}
-
-
-class MethodStub(object):
-    def __init__(self, dtype, form):
-        self.dtype = dtype
-        self.form = form
-
-    def __call__(self, *args, **kwargs):
-        raise RuntimeError(f"cannot call the instance method {self.form} from a static context")
-
-    def method(self, header, name):
-        return self.dtype(header, self.form, name)
+            return {str(URI(parents[0])): to_json(form_of(cls))}
